@@ -379,19 +379,20 @@ register_scalar_function <- function(name, fun, in_type, out_type,
   RegisterScalarUDF(name, scalar_function)
 
   # register with dplyr binding (enables its use in mutate(), filter(), etc.)
+  binding_fun <- function(...) build_expr(name, ...)
+
+  # inject the value of `name` into the expression to avoid saving this
+  # execution environment in the binding, which eliminates a warning when the
+  # same binding is registered twice
+  body(binding_fun) <- expr_substitute(body(binding_fun), sym("name"), name)
+  environment(binding_fun) <- asNamespace("arrow")
+
   register_binding(
     name,
-    function(...) build_expr(name, ...),
+    binding_fun,
     update_cache = TRUE
   )
 
-  # User-defined functions require some special handling
-  # in the query engine which currently require an opt-in using
-  # the R_ARROW_COLLECT_WITH_UDF environment variable while this
-  # behaviour is stabilized.
-  # TODO(ARROW-17178) remove the need for this!
-  Sys.setenv(R_ARROW_COLLECT_WITH_UDF = "true")
-
   invisible(NULL)
 }
 
