@@ -58,6 +58,8 @@ FLOAT_TYPES <- c("float16", "float32", "float64", "halffloat", "float", "double"
 
 #' Infer the arrow Array type from an R object
 #'
+#' [type()] is deprecated in favor of [infer_type()].
+#'
 #' @param x an R object (usually a vector) to be converted to an [Array] or
 #'   [ChunkedArray].
 #' @param ... Passed to S3 methods
@@ -109,6 +111,20 @@ infer_type.default <- function(x, ..., from_array_infer_type = FALSE) {
   }
 }
 
+#' @export
+infer_type.vctrs_list_of <- function(x, ...) {
+  list_of(infer_type(attr(x, "ptype")))
+}
+
+#' @export
+infer_type.blob <- function(x, ...) {
+  if (sum(lengths(x)) > .Machine$integer.max) {
+    large_binary()
+  } else {
+    binary()
+  }
+}
+
 #' @export
 infer_type.ArrowDatum <- function(x, ...) x$type
 
