@@ -18,6 +18,7 @@
 #include "./arrow_types.h"
 
 #include <arrow/array/array_base.h>
+#include <arrow/builder.h>
 #include <arrow/table.h>
 #include <arrow/util/byte_size.h>
 #include <arrow/util/key_value_metadata.h>
@@ -302,6 +303,37 @@ std::shared_ptr<arrow::Table> Table__from_record_batches(
   return tab;
 }
 
+// [[arrow::export]]
+std::shared_ptr<arrow::Table> Table__from_schema(SEXP schema_sxp) {
+  auto schema = cpp11::as_cpp<std::shared_ptr<arrow::Schema>>(schema_sxp);
+
+  int num_fields = schema->num_fields();
+
+  std::vector<std::shared_ptr<arrow::Array>> columns;
+
+  for (int i = 0; i < num_fields; i++) {
+    bool is_extension_type = schema->field(i)->type()->name() == "extension";
+    std::shared_ptr<arrow::DataType> type;
+
+    // need to handle extension types a bit differently
+    if (is_extension_type) {
+      // TODO: ARROW-18043 - update this to properly construct extension types instead of
+      // converting to null
+      type = arrow::null();
+    } else {
+      type = schema->field(i)->type();
+    }
+
+    std::shared_ptr<arrow::Array> array;
+    std::unique_ptr<arrow::ArrayBuilder> type_builder;
+    StopIfNotOk(arrow::MakeBuilder(gc_memory_pool(), type, &type_builder));
+    StopIfNotOk(type_builder->Finish(&array));
+    columns.push_back(array);
+  }
+
+  return (arrow::Table::Make(schema, std::move(columns)));
+}
+
 // [[arrow::export]]
 r_vec_size Table__ReferencedBufferSize(const std::shared_ptr<arrow::Table>& table) {
   return r_vec_size(ValueOrStop(arrow::util::ReferencedBufferSize(*table)));
