@@ -112,8 +112,6 @@ test_that("register_scalar_function() adds a compute function to the registry",
       dplyr::collect(),
     tibble::tibble(a = 1L, b = 32.0)
   )
-
-  Sys.unsetenv("R_ARROW_COLLECT_WITH_UDF")
 })
 
 test_that("arrow_scalar_function() with bad return type errors", {
@@ -125,9 +123,7 @@ test_that("arrow_scalar_function() with bad return type errors", {
     int32(),
     float64()
   )
-  on.exit(
-    unregister_binding("times_32_bad_return_type_array", update_cache = TRUE)
-  )
+  on.exit(unregister_binding("times_32_bad_return_type_array", update_cache = TRUE))
 
   expect_error(
     call_function("times_32_bad_return_type_array", Array$create(1L)),
@@ -140,20 +136,15 @@ test_that("arrow_scalar_function() with bad return type errors", {
     int32(),
     float64()
   )
-  on.exit(
-    unregister_binding("times_32_bad_return_type_scalar", update_cache = TRUE)
-  )
+  on.exit(unregister_binding("times_32_bad_return_type_scalar", update_cache = TRUE))
 
   expect_error(
     call_function("times_32_bad_return_type_scalar", Array$create(1L)),
     "Expected return Array or Scalar with type 'double'"
   )
-
-  # TODO(ARROW-17178) remove the need for this!
-  Sys.unsetenv("R_ARROW_COLLECT_WITH_UDF")
 })
 
-test_that("register_user_defined_function() can register multiple kernels", {
+test_that("register_scalar_function() can register multiple kernels", {
   skip_if_not(CanRunWithCapturedR())
 
   register_scalar_function(
@@ -181,7 +172,7 @@ test_that("register_user_defined_function() can register multiple kernels", {
   )
 })
 
-test_that("register_user_defined_function() errors for unsupported specifications", {
+test_that("register_scalar_function() errors for unsupported specifications", {
   expect_error(
     register_scalar_function(
       "no_kernels",
@@ -211,9 +202,6 @@ test_that("register_user_defined_function() errors for unsupported specification
     ),
     "Kernels for user-defined function must accept the same number of arguments"
   )
-
-  # TODO(ARROW-17178) remove the need for this!
-  Sys.unsetenv("R_ARROW_COLLECT_WITH_UDF")
 })
 
 test_that("user-defined functions work during multi-threaded execution", {
@@ -269,12 +257,9 @@ test_that("user-defined functions work during multi-threaded execution", {
     dplyr::collect()
 
   expect_identical(result2$fun_result, example_df$value * 32)
-
-  # TODO(ARROW-17178) remove the need for this!
-  Sys.unsetenv("R_ARROW_COLLECT_WITH_UDF")
 })
 
-test_that("user-defined error when called from an unsupported context", {
+test_that("nested exec plans can contain user-defined functions", {
   skip_if_not_available("dataset")
   skip_if_not(CanRunWithCapturedR())
 
@@ -301,27 +286,35 @@ test_that("user-defined error when called from an unsupported context", {
       dplyr::collect()
   }
 
-  if (identical(tolower(Sys.info()[["sysname"]]), "windows")) {
-    expect_equal(
-      stream_plan_with_udf(),
-      record_batch(a = 1:1000) %>%
-        dplyr::mutate(b = times_32(a)) %>%
-        dplyr::collect(as_data_frame = FALSE)
-    )
+  expect_equal(
+    stream_plan_with_udf(),
+    record_batch(a = 1:1000) %>%
+      dplyr::mutate(b = times_32(a)) %>%
+      dplyr::collect(as_data_frame = FALSE)
+  )
 
-    result <- collect_plan_with_head()
-    expect_equal(nrow(result), 11)
-  } else {
-    expect_error(
-      stream_plan_with_udf(),
-      "Call to R \\(.*?\\) from a non-R thread from an unsupported context"
-    )
-    expect_error(
-      collect_plan_with_head(),
-      "Call to R \\(.*?\\) from a non-R thread from an unsupported context"
-    )
-  }
-
-  # TODO(ARROW-17178) remove the need for this!
-  Sys.unsetenv("R_ARROW_COLLECT_WITH_UDF")
+  result <- collect_plan_with_head()
+  expect_equal(nrow(result), 11)
+})
+
+test_that("head() on exec plan containing user-defined functions", {
+  skip_if_not_available("dataset")
+  skip_if_not(CanRunWithCapturedR())
+
+  register_scalar_function(
+    "times_32",
+    function(context, x) x * 32.0,
+    int32(),
+    float64(),
+    auto_convert = TRUE
+  )
+  on.exit(unregister_binding("times_32", update_cache = TRUE))
+
+  result <- record_batch(a = 1:1000) %>%
+    dplyr::mutate(b = times_32(a)) %>%
+    as_record_batch_reader() %>%
+    head(11) %>%
+    dplyr::collect()
+
+  expect_equal(nrow(result), 11)
 })
