@@ -391,14 +391,18 @@ test_that("map type works as expected", {
 })
 
 test_that("map type validates arguments", {
-  expect_error(map_of(field("key", int32(), nullable = TRUE), utf8()),
-               "cannot be nullable")
+  expect_error(
+    map_of(field("key", int32(), nullable = TRUE), utf8()),
+    "cannot be nullable"
+  )
   expect_error(map_of(1L, utf8()), "must be a DataType or Field")
   expect_error(map_of(int32(), 1L), "must be a DataType or Field")
 
   # field construction
-  ty <- map_of(field("the_keys", int32(), nullable = FALSE),
-               field("my_values", utf8(), nullable = FALSE))
+  ty <- map_of(
+    field("the_keys", int32(), nullable = FALSE),
+    field("my_values", utf8(), nullable = FALSE)
+  )
   expect_equal(ty$key_field$name, "the_keys")
   expect_equal(ty$item_field$name, "my_values")
   expect_equal(ty$key_field$nullable, FALSE)
@@ -589,16 +593,16 @@ test_that("DataType$code()", {
   expect_code_roundtrip(dictionary(index_type = int8(), value_type = large_utf8()))
   expect_code_roundtrip(dictionary(index_type = int8(), ordered = TRUE))
 
-  skip("until rlang 1.0")
-  expect_snapshot({
-    (expect_error(
-      DayTimeInterval__initialize()$code()
-    ))
-    (expect_error(
-      struct(a = DayTimeInterval__initialize())$code()
-    ))
-  })
-
+  skip_if(packageVersion("rlang") < 1)
+  # Are these unsupported for a reason?
+  expect_error(
+    eval(DayTimeInterval__initialize()$code()),
+    "Unsupported type"
+  )
+  expect_error(
+    eval(struct(a = DayTimeInterval__initialize())$code()),
+    "Unsupported type"
+  )
 })
 
 test_that("as_data_type() works for DataType", {
