@@ -22,9 +22,8 @@
 #include <string>
 #include <vector>
 
-#include "arrow/compute/exec/key_encode.h"  // for KeyColumnMetadata
-#include "arrow/type.h"                     // for DataType, FieldRef, Field and Schema
-#include "arrow/util/mutex.h"
+#include "arrow/compute/light_array.h"  // for KeyColumnMetadata
+#include "arrow/type.h"                 // for DataType, FieldRef, Field and Schema
 
 namespace arrow {
 
@@ -79,16 +78,28 @@ class SchemaProjectionMaps {
 
   int num_cols(ProjectionIdEnum schema_handle) const {
     int id = schema_id(schema_handle);
-    return static_cast<int>(schemas_[id].second.size());
+    return static_cast<int>(schemas_[id].second.data_types.size());
+  }
+
+  bool is_empty(ProjectionIdEnum schema_handle) const {
+    return num_cols(schema_handle) == 0;
   }
 
   const std::string& field_name(ProjectionIdEnum schema_handle, int field_id) const {
-    return field(schema_handle, field_id).field_name;
+    int id = schema_id(schema_handle);
+    return schemas_[id].second.field_names[field_id];
   }
 
   const std::shared_ptr<DataType>& data_type(ProjectionIdEnum schema_handle,
                                              int field_id) const {
-    return field(schema_handle, field_id).data_type;
+    int id = schema_id(schema_handle);
+    return schemas_[id].second.data_types[field_id];
+  }
+
+  const std::vector<std::shared_ptr<DataType>>& data_types(
+      ProjectionIdEnum schema_handle) const {
+    int id = schema_id(schema_handle);
+    return schemas_[id].second.data_types;
   }
 
   SchemaProjectionMap map(ProjectionIdEnum from, ProjectionIdEnum to) const {
@@ -102,22 +113,24 @@ class SchemaProjectionMaps {
   }
 
  protected:
-  struct FieldInfo {
-    int field_path;
-    std::string field_name;
-    std::shared_ptr<DataType> data_type;
+  struct FieldInfos {
+    std::vector<int> field_paths;
+    std::vector<std::string> field_names;
+    std::vector<std::shared_ptr<DataType>> data_types;
   };
 
   Status RegisterSchema(ProjectionIdEnum handle, const Schema& schema) {
-    std::vector<FieldInfo> out_fields;
+    FieldInfos out_fields;
     const FieldVector& in_fields = schema.fields();
-    out_fields.resize(in_fields.size());
+    out_fields.field_paths.resize(in_fields.size());
+    out_fields.field_names.resize(in_fields.size());
+    out_fields.data_types.resize(in_fields.size());
     for (size_t i = 0; i < in_fields.size(); ++i) {
       const std::string& name = in_fields[i]->name();
       const std::shared_ptr<DataType>& type = in_fields[i]->type();
-      out_fields[i].field_path = static_cast<int>(i);
-      out_fields[i].field_name = name;
-      out_fields[i].data_type = type;
+      out_fields.field_paths[i] = static_cast<int>(i);
+      out_fields.field_names[i] = name;
+      out_fields.data_types[i] = type;
     }
     schemas_.push_back(std::make_pair(handle, out_fields));
     return Status::OK();
@@ -126,17 +139,19 @@ class SchemaProjectionMaps {
   Status RegisterProjectedSchema(ProjectionIdEnum handle,
                                  const std::vector<FieldRef>& selected_fields,
                                  const Schema& full_schema) {
-    std::vector<FieldInfo> out_fields;
+    FieldInfos out_fields;
     const FieldVector& in_fields = full_schema.fields();
-    out_fields.resize(selected_fields.size());
+    out_fields.field_paths.resize(selected_fields.size());
+    out_fields.field_names.resize(selected_fields.size());
+    out_fields.data_types.resize(selected_fields.size());
     for (size_t i = 0; i < selected_fields.size(); ++i) {
       // All fields must be found in schema without ambiguity
       ARROW_ASSIGN_OR_RAISE(auto match, selected_fields[i].FindOne(full_schema));
       const std::string& name = in_fields[match[0]]->name();
       const std::shared_ptr<DataType>& type = in_fields[match[0]]->type();
-      out_fields[i].field_path = match[0];
-      out_fields[i].field_name = name;
-      out_fields[i].data_type = type;
+      out_fields.field_paths[i] = match[0];
+      out_fields.field_names[i] = name;
+      out_fields.data_types[i] = type;
     }
     schemas_.push_back(std::make_pair(handle, out_fields));
     return Status::OK();
@@ -163,15 +178,9 @@ class SchemaProjectionMaps {
     return -1;
   }
 
-  const FieldInfo& field(ProjectionIdEnum schema_handle, int field_id) const {
-    int id = schema_id(schema_handle);
-    const std::vector<FieldInfo>& field_infos = schemas_[id].second;
-    return field_infos[field_id];
-  }
-
   void GenerateMapForProjection(int id_proj, int id_base) {
-    int num_cols_proj = static_cast<int>(schemas_[id_proj].second.size());
-    int num_cols_base = static_cast<int>(schemas_[id_base].second.size());
+    int num_cols_proj = static_cast<int>(schemas_[id_proj].second.data_types.size());
+    int num_cols_base = static_cast<int>(schemas_[id_base].second.data_types.size());
 
     std::vector<int>& mapping = mappings_[id_proj];
     std::vector<int>& inverse_mapping = inverse_mappings_[id_proj];
@@ -183,15 +192,15 @@ class SchemaProjectionMaps {
         mapping[i] = inverse_mapping[i] = i;
       }
     } else {
-      const std::vector<FieldInfo>& fields_proj = schemas_[id_proj].second;
-      const std::vector<FieldInfo>& fields_base = schemas_[id_base].second;
+      const FieldInfos& fields_proj = schemas_[id_proj].second;
+      const FieldInfos& fields_base = schemas_[id_base].second;
       for (int i = 0; i < num_cols_base; ++i) {
         inverse_mapping[i] = SchemaProjectionMap::kMissingField;
       }
       for (int i = 0; i < num_cols_proj; ++i) {
         int field_id = SchemaProjectionMap::kMissingField;
         for (int j = 0; j < num_cols_base; ++j) {
-          if (fields_proj[i].field_path == fields_base[j].field_path) {
+          if (fields_proj.field_paths[i] == fields_base.field_paths[j]) {
             field_id = j;
             // If there are multiple matches for the same input field,
             // it will be mapped to the first match.
@@ -206,10 +215,12 @@ class SchemaProjectionMaps {
   }
 
   // vector used as a mapping from ProjectionIdEnum to fields
-  std::vector<std::pair<ProjectionIdEnum, std::vector<FieldInfo>>> schemas_;
+  std::vector<std::pair<ProjectionIdEnum, FieldInfos>> schemas_;
   std::vector<std::vector<int>> mappings_;
   std::vector<std::vector<int>> inverse_mappings_;
 };
 
+using HashJoinProjectionMaps = SchemaProjectionMaps<HashJoinProjection>;
+
 }  // namespace compute
 }  // namespace arrow
