@@ -18,6 +18,7 @@
 #include "arrow/dataset/dataset_writer.h"
 
 #include <deque>
+#include <memory>
 #include <mutex>
 #include <unordered_map>
 
@@ -31,12 +32,13 @@
 #include "arrow/util/string.h"
 
 namespace arrow {
+using internal::Executor;
 namespace dataset {
 namespace internal {
 
 namespace {
 
-constexpr util::string_view kIntegerToken = "{i}";
+constexpr std::string_view kIntegerToken = "{i}";
 
 class Throttle {
  public:
@@ -112,18 +114,35 @@ struct DatasetWriterState {
   std::mutex visitors_mutex;
 };
 
-class DatasetWriterFileQueue : public util::AsyncDestroyable {
+Result<std::shared_ptr<FileWriter>> OpenWriter(
+    const FileSystemDatasetWriteOptions& write_options, std::shared_ptr<Schema> schema,
+    const std::string& filename) {
+  ARROW_ASSIGN_OR_RAISE(std::shared_ptr<io::OutputStream> out_stream,
+                        write_options.filesystem->OpenOutputStream(filename));
+  return write_options.format()->MakeWriter(std::move(out_stream), std::move(schema),
+                                            write_options.file_write_options,
+                                            {write_options.filesystem, filename});
+}
+
+class DatasetWriterFileQueue {
  public:
-  explicit DatasetWriterFileQueue(const Future<std::shared_ptr<FileWriter>>& writer_fut,
+  explicit DatasetWriterFileQueue(const std::shared_ptr<Schema>& schema,
                                   const FileSystemDatasetWriteOptions& options,
                                   DatasetWriterState* writer_state)
-      : options_(options), writer_state_(writer_state) {
-    // If this AddTask call fails (e.g. we're given an already failing future) then we
-    // will get the error later when we try and write to it.
-    ARROW_UNUSED(file_tasks_.AddTask([this, writer_fut] {
-      return writer_fut.Then(
-          [this](const std::shared_ptr<FileWriter>& writer) { writer_ = writer; });
-    }));
+      : options_(options), schema_(schema), writer_state_(writer_state) {}
+
+  void Start(std::shared_ptr<util::AsyncTaskScheduler> scheduler,
+             const std::string& filename) {
+    scheduler_ = scheduler;
+    // Because the scheduler runs one task at a time we know the writer will
+    // be opened before any attempt to write
+    scheduler->AddSimpleTask([this, filename] {
+      Executor* io_executor = options_.filesystem->io_context().executor();
+      return DeferNotOk(io_executor->Submit([this, filename]() {
+        ARROW_ASSIGN_OR_RAISE(writer_, OpenWriter(options_, schema_, filename));
+        return Status::OK();
+      }));
+    });
   }
 
   Result<std::shared_ptr<RecordBatch>> PopStagedBatch() {
@@ -155,20 +174,17 @@ class DatasetWriterFileQueue : public util::AsyncDestroyable {
     return table->CombineChunksToBatch();
   }
 
-  Status ScheduleBatch(std::shared_ptr<RecordBatch> batch) {
-    struct WriteTask {
-      Future<> operator()() { return self->WriteNext(std::move(batch)); }
-      DatasetWriterFileQueue* self;
-      std::shared_ptr<RecordBatch> batch;
-    };
-    return file_tasks_.AddTask(WriteTask{this, std::move(batch)});
+  void ScheduleBatch(std::shared_ptr<RecordBatch> batch) {
+    scheduler_->AddSimpleTask([self = this, batch = std::move(batch)]() {
+      return self->WriteNext(std::move(batch));
+    });
   }
 
   Result<int64_t> PopAndDeliverStagedBatch() {
     ARROW_ASSIGN_OR_RAISE(std::shared_ptr<RecordBatch> next_batch, PopStagedBatch());
     int64_t rows_popped = next_batch->num_rows();
     rows_currently_staged_ -= next_batch->num_rows();
-    ARROW_RETURN_NOT_OK(ScheduleBatch(std::move(next_batch)));
+    ScheduleBatch(std::move(next_batch));
     return rows_popped;
   }
 
@@ -188,29 +204,29 @@ class DatasetWriterFileQueue : public util::AsyncDestroyable {
     return Status::OK();
   }
 
-  Future<> DoDestroy() override {
+  Status Finish() {
     writer_state_->staged_rows_count -= rows_currently_staged_;
     while (!staged_batches_.empty()) {
       RETURN_NOT_OK(PopAndDeliverStagedBatch());
     }
-    return file_tasks_.End().Then([this] { return DoFinish(); });
+    // At this point all write tasks have been added.  Because the scheduler
+    // is a 1-task FIFO we know this task will run at the very end and can
+    // add it now.
+    scheduler_->AddSimpleTask([this] { return DoFinish(); });
+    scheduler_.reset();
+    return Status::OK();
   }
 
  private:
   Future<> WriteNext(std::shared_ptr<RecordBatch> next) {
-    struct WriteTask {
-      Status operator()() {
-        int64_t rows_to_release = batch->num_rows();
-        Status status = self->writer_->Write(batch);
-        self->writer_state_->rows_in_flight_throttle.Release(rows_to_release);
-        return status;
-      }
-      DatasetWriterFileQueue* self;
-      std::shared_ptr<RecordBatch> batch;
-    };
     // May want to prototype / measure someday pushing the async write down further
     return DeferNotOk(options_.filesystem->io_context().executor()->Submit(
-        WriteTask{this, std::move(next)}));
+        [self = this, batch = std::move(next)]() {
+          int64_t rows_to_release = batch->num_rows();
+          Status status = self->writer_->Write(batch);
+          self->writer_state_->rows_in_flight_throttle.Release(rows_to_release);
+          return status;
+        }));
   }
 
   Future<> DoFinish() {
@@ -225,13 +241,14 @@ class DatasetWriterFileQueue : public util::AsyncDestroyable {
   }
 
   const FileSystemDatasetWriteOptions& options_;
+  const std::shared_ptr<Schema>& schema_;
   DatasetWriterState* writer_state_;
   std::shared_ptr<FileWriter> writer_;
   // Batches are accumulated here until they are large enough to write out at which
   // point they are merged together and added to write_queue_
   std::deque<std::shared_ptr<RecordBatch>> staged_batches_;
   uint64_t rows_currently_staged_ = 0;
-  util::SerializedAsyncTaskGroup file_tasks_;
+  std::shared_ptr<util::AsyncTaskScheduler> scheduler_ = nullptr;
 };
 
 struct WriteTask {
@@ -239,12 +256,15 @@ struct WriteTask {
   uint64_t num_rows;
 };
 
-class DatasetWriterDirectoryQueue : public util::AsyncDestroyable {
+class DatasetWriterDirectoryQueue {
  public:
-  DatasetWriterDirectoryQueue(std::string directory, std::shared_ptr<Schema> schema,
+  DatasetWriterDirectoryQueue(util::AsyncTaskScheduler* scheduler, std::string directory,
+                              std::string prefix, std::shared_ptr<Schema> schema,
                               const FileSystemDatasetWriteOptions& write_options,
                               DatasetWriterState* writer_state)
-      : directory_(std::move(directory)),
+      : scheduler_(scheduler),
+        directory_(std::move(directory)),
+        prefix_(std::move(prefix)),
         schema_(std::move(schema)),
         write_options_(write_options),
         writer_state_(writer_state) {}
@@ -284,102 +304,123 @@ class DatasetWriterDirectoryQueue : public util::AsyncDestroyable {
     if (!basename) {
       return Status::Invalid("string interpolation of basename template failed");
     }
-
-    return fs::internal::ConcatAbstractPath(directory_, *basename);
+    return fs::internal::ConcatAbstractPath(directory_, prefix_ + *basename);
   }
 
   Status FinishCurrentFile() {
     if (latest_open_file_) {
+      ARROW_RETURN_NOT_OK(latest_open_file_->Finish());
       latest_open_file_ = nullptr;
     }
     rows_written_ = 0;
     return GetNextFilename().Value(&current_filename_);
   }
 
-  Result<std::shared_ptr<FileWriter>> OpenWriter(const std::string& filename) {
-    ARROW_ASSIGN_OR_RAISE(std::shared_ptr<io::OutputStream> out_stream,
-                          write_options_.filesystem->OpenOutputStream(filename));
-    return write_options_.format()->MakeWriter(std::move(out_stream), schema_,
-                                               write_options_.file_write_options,
-                                               {write_options_.filesystem, filename});
-  }
-
-  Result<std::shared_ptr<DatasetWriterFileQueue>> OpenFileQueue(
-      const std::string& filename) {
-    Future<std::shared_ptr<FileWriter>> file_writer_fut =
-        init_future_.Then([this, filename] {
-          ::arrow::internal::Executor* io_executor =
-              write_options_.filesystem->io_context().executor();
-          return DeferNotOk(
-              io_executor->Submit([this, filename]() { return OpenWriter(filename); }));
-        });
-    auto file_queue = util::MakeSharedAsync<DatasetWriterFileQueue>(
-        file_writer_fut, write_options_, writer_state_);
-    RETURN_NOT_OK(task_group_.AddTask(file_queue->on_closed().Then(
-        [this] { writer_state_->open_files_throttle.Release(1); },
-        [this](const Status& err) {
-          writer_state_->open_files_throttle.Release(1);
-          return err;
-        })));
-    return file_queue;
+  Result<DatasetWriterFileQueue*> OpenFileQueue(const std::string& filename) {
+    auto file_queue =
+        std::make_unique<DatasetWriterFileQueue>(schema_, write_options_, writer_state_);
+    DatasetWriterFileQueue* file_queue_view = file_queue.get();
+    std::unique_ptr<util::AsyncTaskScheduler::Throttle> throttle =
+        util::AsyncTaskScheduler::MakeThrottle(1);
+    util::AsyncTaskScheduler::Throttle* throttle_view = throttle.get();
+    auto file_finish_task = [self = this, file_queue = std::move(file_queue),
+                             throttle = std::move(throttle)](Status) {
+      self->writer_state_->open_files_throttle.Release(1);
+      return Status::OK();
+    };
+    std::shared_ptr<util::AsyncTaskScheduler> file_scheduler =
+        scheduler_->MakeSubScheduler(std::move(file_finish_task), throttle_view);
+    if (init_future_.is_valid()) {
+      file_scheduler->AddSimpleTask(
+          [init_future = init_future_]() { return init_future; });
+    }
+    file_queue_view->Start(std::move(file_scheduler), filename);
+    return file_queue_view;
   }
 
   uint64_t rows_written() const { return rows_written_; }
 
   void PrepareDirectory() {
-    init_future_ = DeferNotOk(write_options_.filesystem->io_context().executor()->Submit(
-        [this]() { return write_options_.filesystem->CreateDir(directory_); }));
+    if (directory_.empty() || !write_options_.create_dir) {
+      return;
+    }
+    init_future_ = Future<>::Make();
+    auto create_dir_cb = [this] {
+      return DeferNotOk(write_options_.filesystem->io_context().executor()->Submit(
+          [this]() { return write_options_.filesystem->CreateDir(directory_); }));
+    };
+    // We need to notify waiters whether the directory succeeded or failed.
+    auto notify_waiters_cb = [this] { init_future_.MarkFinished(); };
+    auto notify_waiters_on_err_cb = [this](const Status& err) {
+      // If there is an error the scheduler will abort but that takes some time
+      // and we don't want to start writing in the meantime so we send an error to the
+      // file writing queue and return the error.
+      init_future_.MarkFinished(err);
+      return err;
+    };
+    std::function<Future<>()> init_task;
     if (write_options_.existing_data_behavior ==
         ExistingDataBehavior::kDeleteMatchingPartitions) {
-      init_future_ = init_future_.Then([this]() {
-        return write_options_.filesystem->DeleteDirContentsAsync(directory_);
-      });
+      init_task = [this, create_dir_cb, notify_waiters_cb, notify_waiters_on_err_cb] {
+        return write_options_.filesystem
+            ->DeleteDirContentsAsync(directory_,
+                                     /*missing_dir_ok=*/true)
+            .Then(create_dir_cb)
+            .Then(notify_waiters_cb, notify_waiters_on_err_cb);
+      };
+    } else {
+      init_task = [create_dir_cb, notify_waiters_cb, notify_waiters_on_err_cb] {
+        return create_dir_cb().Then(notify_waiters_cb, notify_waiters_on_err_cb);
+      };
     }
+    scheduler_->AddSimpleTask(std::move(init_task));
   }
 
-  static Result<std::unique_ptr<DatasetWriterDirectoryQueue,
-                                util::DestroyingDeleter<DatasetWriterDirectoryQueue>>>
-  Make(util::AsyncTaskGroup* task_group,
-       const FileSystemDatasetWriteOptions& write_options,
-       DatasetWriterState* writer_state, std::shared_ptr<Schema> schema,
-       std::string dir) {
-    auto dir_queue = util::MakeUniqueAsync<DatasetWriterDirectoryQueue>(
-        std::move(dir), std::move(schema), write_options, writer_state);
-    RETURN_NOT_OK(task_group->AddTask(dir_queue->on_closed()));
+  static Result<std::unique_ptr<DatasetWriterDirectoryQueue>> Make(
+      util::AsyncTaskScheduler* scheduler,
+      const FileSystemDatasetWriteOptions& write_options,
+      DatasetWriterState* writer_state, std::shared_ptr<Schema> schema,
+      std::string directory, std::string prefix) {
+    auto dir_queue = std::make_unique<DatasetWriterDirectoryQueue>(
+        scheduler, std::move(directory), std::move(prefix), std::move(schema),
+        write_options, writer_state);
     dir_queue->PrepareDirectory();
     ARROW_ASSIGN_OR_RAISE(dir_queue->current_filename_, dir_queue->GetNextFilename());
     // std::move required to make RTools 3.5 mingw compiler happy
     return std::move(dir_queue);
   }
 
-  Future<> DoDestroy() override {
-    latest_open_file_.reset();
-    return task_group_.End();
+  Status Finish() {
+    if (latest_open_file_) {
+      return latest_open_file_->Finish();
+    }
+    return Status::OK();
   }
 
  private:
-  util::AsyncTaskGroup task_group_;
+  util::AsyncTaskScheduler* scheduler_ = nullptr;
   std::string directory_;
+  std::string prefix_;
   std::shared_ptr<Schema> schema_;
   const FileSystemDatasetWriteOptions& write_options_;
   DatasetWriterState* writer_state_;
   Future<> init_future_;
   std::string current_filename_;
-  std::shared_ptr<DatasetWriterFileQueue> latest_open_file_;
+  DatasetWriterFileQueue* latest_open_file_ = nullptr;
   uint64_t rows_written_ = 0;
   uint32_t file_counter_ = 0;
 };
 
-Status ValidateBasenameTemplate(util::string_view basename_template) {
-  if (basename_template.find(fs::internal::kSep) != util::string_view::npos) {
+Status ValidateBasenameTemplate(std::string_view basename_template) {
+  if (basename_template.find(fs::internal::kSep) != std::string_view::npos) {
     return Status::Invalid("basename_template contained '/'");
   }
   size_t token_start = basename_template.find(kIntegerToken);
-  if (token_start == util::string_view::npos) {
+  if (token_start == std::string_view::npos) {
     return Status::Invalid("basename_template did not contain '", kIntegerToken, "'");
   }
   size_t next_token_start = basename_template.find(kIntegerToken, token_start + 1);
-  if (next_token_start != util::string_view::npos) {
+  if (next_token_start != std::string_view::npos) {
     return Status::Invalid("basename_template contained '", kIntegerToken,
                            "' more than once");
   }
@@ -388,11 +429,18 @@ Status ValidateBasenameTemplate(util::string_view basename_template) {
 
 Status ValidateOptions(const FileSystemDatasetWriteOptions& options) {
   ARROW_RETURN_NOT_OK(ValidateBasenameTemplate(options.basename_template));
+  if (!options.file_write_options) {
+    return Status::Invalid("Must provide file_write_options");
+  }
+  if (!options.filesystem) {
+    return Status::Invalid("Must provide filesystem");
+  }
   if (options.max_rows_per_group <= 0) {
     return Status::Invalid("max_rows_per_group must be a positive number");
   }
   if (options.max_rows_per_group < options.min_rows_per_group) {
-    return Status::Invalid("max_rows_per_group must be less than min_rows_per_group");
+    return Status::Invalid(
+        "min_rows_per_group must be less than or equal to max_rows_per_group");
   }
   if (options.max_rows_per_file > 0 &&
       options.max_rows_per_file < options.max_rows_per_group) {
@@ -413,7 +461,7 @@ Status EnsureDestinationValid(const FileSystemDatasetWriteOptions& options) {
       // If the path doesn't exist then continue
       return Status::OK();
     }
-    if (maybe_files->size() > 1) {
+    if (maybe_files->size() > 0) {
       return Status::Invalid(
           "Could not write to ", options.base_dir,
           " as the directory is not empty and existing_data_behavior is to error");
@@ -431,28 +479,36 @@ uint64_t CalculateMaxRowsStaged(uint64_t max_rows_queued) {
 
 }  // namespace
 
-class DatasetWriter::DatasetWriterImpl : public util::AsyncDestroyable {
+class DatasetWriter::DatasetWriterImpl {
  public:
-  DatasetWriterImpl(FileSystemDatasetWriteOptions write_options, uint64_t max_rows_queued)
-      : write_options_(std::move(write_options)),
+  DatasetWriterImpl(FileSystemDatasetWriteOptions write_options,
+                    util::AsyncTaskScheduler* scheduler, uint64_t max_rows_queued)
+      : scheduler_(scheduler),
+        write_options_(std::move(write_options)),
         writer_state_(max_rows_queued, write_options_.max_open_files,
                       CalculateMaxRowsStaged(max_rows_queued)) {}
 
   Future<> WriteRecordBatch(std::shared_ptr<RecordBatch> batch,
-                            const std::string& directory) {
-    RETURN_NOT_OK(CheckError());
+                            const std::string& directory, const std::string& prefix) {
     if (batch->num_rows() == 0) {
       return Future<>::MakeFinished();
     }
     if (!directory.empty()) {
       auto full_path =
           fs::internal::ConcatAbstractPath(write_options_.base_dir, directory);
-      return DoWriteRecordBatch(std::move(batch), full_path);
+      return DoWriteRecordBatch(std::move(batch), full_path, prefix);
     } else {
-      return DoWriteRecordBatch(std::move(batch), write_options_.base_dir);
+      return DoWriteRecordBatch(std::move(batch), write_options_.base_dir, prefix);
     }
   }
 
+  Status Finish() {
+    for (const auto& directory_queue : directory_queues_) {
+      ARROW_RETURN_NOT_OK(directory_queue.second->Finish());
+    }
+    return Status::OK();
+  }
+
  protected:
   Status CloseLargestFile() {
     std::shared_ptr<DatasetWriterDirectoryQueue> largest = nullptr;
@@ -468,13 +524,15 @@ class DatasetWriter::DatasetWriterImpl : public util::AsyncDestroyable {
   }
 
   Future<> DoWriteRecordBatch(std::shared_ptr<RecordBatch> batch,
-                              const std::string& directory) {
+                              const std::string& directory, const std::string& prefix) {
     ARROW_ASSIGN_OR_RAISE(
         auto dir_queue_itr,
         ::arrow::internal::GetOrInsertGenerated(
-            &directory_queues_, directory, [this, &batch](const std::string& dir) {
-              return DatasetWriterDirectoryQueue::Make(
-                  &task_group_, write_options_, &writer_state_, batch->schema(), dir);
+            &directory_queues_, directory + prefix,
+            [this, &batch, &directory, &prefix](const std::string& key) {
+              return DatasetWriterDirectoryQueue::Make(scheduler_, write_options_,
+                                                       &writer_state_, batch->schema(),
+                                                       directory, prefix);
             }));
     std::shared_ptr<DatasetWriterDirectoryQueue> dir_queue = dir_queue_itr->second;
     Future<> backpressure;
@@ -505,28 +563,14 @@ class DatasetWriter::DatasetWriterImpl : public util::AsyncDestroyable {
     }
 
     if (batch) {
-      return backpressure.Then(
-          [this, batch, directory] { return DoWriteRecordBatch(batch, directory); });
+      return backpressure.Then([this, batch, directory, prefix] {
+        return DoWriteRecordBatch(batch, directory, prefix);
+      });
     }
     return Future<>::MakeFinished();
   }
 
-  void SetError(Status st) {
-    std::lock_guard<std::mutex> lg(mutex_);
-    err_ = std::move(st);
-  }
-
-  Status CheckError() {
-    std::lock_guard<std::mutex> lg(mutex_);
-    return err_;
-  }
-
-  Future<> DoDestroy() override {
-    directory_queues_.clear();
-    return task_group_.End().Then([this] { return err_; });
-  }
-
-  util::AsyncTaskGroup task_group_;
+  util::AsyncTaskScheduler* scheduler_ = nullptr;
   FileSystemDatasetWriteOptions write_options_;
   DatasetWriterState writer_state_;
   std::unordered_map<std::string, std::shared_ptr<DatasetWriterDirectoryQueue>>
@@ -536,30 +580,29 @@ class DatasetWriter::DatasetWriterImpl : public util::AsyncDestroyable {
 };
 
 DatasetWriter::DatasetWriter(FileSystemDatasetWriteOptions write_options,
+                             util::AsyncTaskScheduler* scheduler,
                              uint64_t max_rows_queued)
-    : impl_(util::MakeUniqueAsync<DatasetWriterImpl>(std::move(write_options),
-                                                     max_rows_queued)) {}
+    : impl_(std::make_unique<DatasetWriterImpl>(std::move(write_options), scheduler,
+                                                max_rows_queued)) {}
 
 Result<std::unique_ptr<DatasetWriter>> DatasetWriter::Make(
-    FileSystemDatasetWriteOptions write_options, uint64_t max_rows_queued) {
+    FileSystemDatasetWriteOptions write_options, util::AsyncTaskScheduler* scheduler,
+    uint64_t max_rows_queued) {
   RETURN_NOT_OK(ValidateOptions(write_options));
   RETURN_NOT_OK(EnsureDestinationValid(write_options));
   return std::unique_ptr<DatasetWriter>(
-      new DatasetWriter(std::move(write_options), max_rows_queued));
+      new DatasetWriter(std::move(write_options), scheduler, max_rows_queued));
 }
 
 DatasetWriter::~DatasetWriter() = default;
 
 Future<> DatasetWriter::WriteRecordBatch(std::shared_ptr<RecordBatch> batch,
-                                         const std::string& directory) {
-  return impl_->WriteRecordBatch(std::move(batch), directory);
+                                         const std::string& directory,
+                                         const std::string& prefix) {
+  return impl_->WriteRecordBatch(std::move(batch), directory, prefix);
 }
 
-Future<> DatasetWriter::Finish() {
-  Future<> finished = impl_->on_closed();
-  impl_.reset();
-  return finished;
-}
+Status DatasetWriter::Finish() { return impl_->Finish(); }
 
 }  // namespace internal
 }  // namespace dataset
