@@ -98,7 +98,7 @@ Result<std::shared_ptr<SchemaManifest>> GetSchemaManifest(
   return manifest;
 }
 
-util::optional<compute::Expression> ColumnChunkStatisticsAsExpression(
+std::optional<compute::Expression> ColumnChunkStatisticsAsExpression(
     const SchemaField& schema_field, const parquet::RowGroupMetaData& metadata) {
   // For the remaining of this function, failure to extract/parse statistics
   // are ignored by returning nullptr. The goal is two fold. First
@@ -107,13 +107,13 @@ util::optional<compute::Expression> ColumnChunkStatisticsAsExpression(
 
   // For now, only leaf (primitive) types are supported.
   if (!schema_field.is_leaf()) {
-    return util::nullopt;
+    return std::nullopt;
   }
 
   auto column_metadata = metadata.ColumnChunk(schema_field.column_index);
   auto statistics = column_metadata->statistics();
   if (statistics == nullptr) {
-    return util::nullopt;
+    return std::nullopt;
   }
 
   const auto& field = schema_field.field;
@@ -126,7 +126,7 @@ util::optional<compute::Expression> ColumnChunkStatisticsAsExpression(
 
   std::shared_ptr<Scalar> min, max;
   if (!StatisticsAsScalars(*statistics, &min, &max).ok()) {
-    return util::nullopt;
+    return std::nullopt;
   }
 
   auto maybe_min = min->CastTo(field->type());
@@ -155,7 +155,7 @@ util::optional<compute::Expression> ColumnChunkStatisticsAsExpression(
     return in_range;
   }
 
-  return util::nullopt;
+  return std::nullopt;
 }
 
 void AddColumnIndices(const SchemaField& schema_field,
@@ -482,17 +482,17 @@ Result<RecordBatchGenerator> ParquetFileFormat::ScanBatchesAsync(
   return generator;
 }
 
-Future<util::optional<int64_t>> ParquetFileFormat::CountRows(
+Future<std::optional<int64_t>> ParquetFileFormat::CountRows(
     const std::shared_ptr<FileFragment>& file, compute::Expression predicate,
     const std::shared_ptr<ScanOptions>& options) {
   auto parquet_file = checked_pointer_cast<ParquetFileFragment>(file);
   if (parquet_file->metadata()) {
     ARROW_ASSIGN_OR_RAISE(auto maybe_count,
                           parquet_file->TryCountRows(std::move(predicate)));
-    return Future<util::optional<int64_t>>::MakeFinished(maybe_count);
+    return Future<std::optional<int64_t>>::MakeFinished(maybe_count);
   } else {
     return DeferNotOk(options->io_context.executor()->Submit(
-        [parquet_file, predicate]() -> Result<util::optional<int64_t>> {
+        [parquet_file, predicate]() -> Result<std::optional<int64_t>> {
           RETURN_NOT_OK(parquet_file->EnsureCompleteMetadata());
           return parquet_file->TryCountRows(predicate);
         }));
@@ -512,7 +512,7 @@ Result<std::shared_ptr<FileFragment>> ParquetFileFormat::MakeFragment(
     std::shared_ptr<Schema> physical_schema) {
   return std::shared_ptr<FileFragment>(new ParquetFileFragment(
       std::move(source), shared_from_this(), std::move(partition_expression),
-      std::move(physical_schema), util::nullopt));
+      std::move(physical_schema), std::nullopt));
 }
 
 //
@@ -573,7 +573,7 @@ ParquetFileFragment::ParquetFileFragment(FileSource source,
                                          std::shared_ptr<FileFormat> format,
                                          compute::Expression partition_expression,
                                          std::shared_ptr<Schema> physical_schema,
-                                         util::optional<std::vector<int>> row_groups)
+                                         std::optional<std::vector<int>> row_groups)
     : FileFragment(std::move(source), std::move(format), std::move(partition_expression),
                    std::move(physical_schema)),
       parquet_format_(checked_cast<ParquetFileFormat&>(*format_)),
@@ -738,7 +738,7 @@ Result<std::vector<compute::Expression>> ParquetFileFragment::TestRowGroups(
   return row_groups;
 }
 
-Result<util::optional<int64_t>> ParquetFileFragment::TryCountRows(
+Result<std::optional<int64_t>> ParquetFileFragment::TryCountRows(
     compute::Expression predicate) {
   DCHECK_NE(metadata_, nullptr);
   if (ExpressionHasFieldRefs(predicate)) {
@@ -757,7 +757,7 @@ Result<util::optional<int64_t>> ParquetFileFragment::TryCountRows(
       // If the row group is entirely excluded, exclude it from the row count
       if (!expressions[i].IsSatisfiable()) continue;
       // Unless the row group is entirely included, bail out of fast path
-      if (expressions[i] != compute::literal(true)) return util::nullopt;
+      if (expressions[i] != compute::literal(true)) return std::nullopt;
       BEGIN_PARQUET_CATCH_EXCEPTIONS
       rows += metadata()->RowGroup((*row_groups_)[i])->num_rows();
       END_PARQUET_CATCH_EXCEPTIONS
