@@ -23,6 +23,7 @@
 #include <cstring>
 #include <memory>
 #include <string>
+#include <string_view>
 #include <type_traits>
 #include <vector>
 
@@ -41,9 +42,8 @@
 #include "arrow/util/bit_util.h"
 #include "arrow/util/checked_cast.h"
 #include "arrow/util/endian.h"
-#include "arrow/util/int_util_internal.h"
+#include "arrow/util/int_util_overflow.h"
 #include "arrow/util/logging.h"
-#include "arrow/util/string_view.h"
 #include "arrow/util/ubsan.h"
 
 #include "parquet/arrow/reader.h"
@@ -303,9 +303,23 @@ Status StatisticsAsScalars(const Statistics& statistics,
 
 namespace {
 
+/// Drop the validity buffer from each chunk.
+///
+/// Used when reading a non-nullable field.
+void ReconstructChunksWithoutNulls(::arrow::ArrayVector* chunks) {
+  for (size_t i = 0; i < chunks->size(); i++) {
+    if ((*chunks)[i]->data()->buffers[0]) {
+      std::shared_ptr<::arrow::ArrayData> data = (*chunks)[i]->data();
+      data->null_count = 0;
+      data->buffers[0] = nullptr;
+      (*chunks)[i] = MakeArray(data);
+    }
+  }
+}
+
 template <typename ArrowType, typename ParquetType>
 Status TransferInt(RecordReader* reader, MemoryPool* pool,
-                   const std::shared_ptr<DataType>& type, Datum* out) {
+                   const std::shared_ptr<Field>& field, Datum* out) {
   using ArrowCType = typename ArrowType::c_type;
   using ParquetCType = typename ParquetType::c_type;
   int64_t length = reader->values_written();
@@ -315,21 +329,35 @@ Status TransferInt(RecordReader* reader, MemoryPool* pool,
   auto values = reinterpret_cast<const ParquetCType*>(reader->values());
   auto out_ptr = reinterpret_cast<ArrowCType*>(data->mutable_data());
   std::copy(values, values + length, out_ptr);
-  *out = std::make_shared<ArrayType<ArrowType>>(
-      type, length, std::move(data), reader->ReleaseIsValid(), reader->null_count());
+  if (field->nullable()) {
+    *out = std::make_shared<ArrayType<ArrowType>>(field->type(), length, std::move(data),
+                                                  reader->ReleaseIsValid(),
+                                                  reader->null_count());
+  } else {
+    *out =
+        std::make_shared<ArrayType<ArrowType>>(field->type(), length, std::move(data),
+                                               /*null_bitmap=*/nullptr, /*null_count=*/0);
+  }
   return Status::OK();
 }
 
 std::shared_ptr<Array> TransferZeroCopy(RecordReader* reader,
-                                        const std::shared_ptr<DataType>& type) {
-  std::vector<std::shared_ptr<Buffer>> buffers = {reader->ReleaseIsValid(),
-                                                  reader->ReleaseValues()};
-  auto data = std::make_shared<::arrow::ArrayData>(type, reader->values_written(),
-                                                   buffers, reader->null_count());
+                                        const std::shared_ptr<Field>& field) {
+  std::shared_ptr<::arrow::ArrayData> data;
+  if (field->nullable()) {
+    std::vector<std::shared_ptr<Buffer>> buffers = {reader->ReleaseIsValid(),
+                                                    reader->ReleaseValues()};
+    data = std::make_shared<::arrow::ArrayData>(field->type(), reader->values_written(),
+                                                std::move(buffers), reader->null_count());
+  } else {
+    std::vector<std::shared_ptr<Buffer>> buffers = {nullptr, reader->ReleaseValues()};
+    data = std::make_shared<::arrow::ArrayData>(field->type(), reader->values_written(),
+                                                std::move(buffers), /*null_count=*/0);
+  }
   return ::arrow::MakeArray(data);
 }
 
-Status TransferBool(RecordReader* reader, MemoryPool* pool, Datum* out) {
+Status TransferBool(RecordReader* reader, bool nullable, MemoryPool* pool, Datum* out) {
   int64_t length = reader->values_written();
 
   const int64_t buffer_size = bit_util::BytesForBits(length);
@@ -346,13 +374,18 @@ Status TransferBool(RecordReader* reader, MemoryPool* pool, Datum* out) {
     }
   }
 
-  *out = std::make_shared<BooleanArray>(length, std::move(data), reader->ReleaseIsValid(),
-                                        reader->null_count());
+  if (nullable) {
+    *out = std::make_shared<BooleanArray>(length, std::move(data),
+                                          reader->ReleaseIsValid(), reader->null_count());
+  } else {
+    *out = std::make_shared<BooleanArray>(length, std::move(data),
+                                          /*null_bitmap=*/nullptr, /*null_count=*/0);
+  }
   return Status::OK();
 }
 
 Status TransferInt96(RecordReader* reader, MemoryPool* pool,
-                     const std::shared_ptr<DataType>& type, Datum* out,
+                     const std::shared_ptr<Field>& field, Datum* out,
                      const ::arrow::TimeUnit::type int96_arrow_time_unit) {
   int64_t length = reader->values_written();
   auto values = reinterpret_cast<const Int96*>(reader->values());
@@ -381,13 +414,19 @@ Status TransferInt96(RecordReader* reader, MemoryPool* pool,
       }
     }
   }
-  *out = std::make_shared<TimestampArray>(type, length, std::move(data),
-                                          reader->ReleaseIsValid(), reader->null_count());
+  if (field->nullable()) {
+    *out =
+        std::make_shared<TimestampArray>(field->type(), length, std::move(data),
+                                         reader->ReleaseIsValid(), reader->null_count());
+  } else {
+    *out = std::make_shared<TimestampArray>(field->type(), length, std::move(data),
+                                            /*null_bitmap=*/nullptr, /*null_count=*/0);
+  }
   return Status::OK();
 }
 
 Status TransferDate64(RecordReader* reader, MemoryPool* pool,
-                      const std::shared_ptr<DataType>& type, Datum* out) {
+                      const std::shared_ptr<Field>& field, Datum* out) {
   int64_t length = reader->values_written();
   auto values = reinterpret_cast<const int32_t*>(reader->values());
 
@@ -399,8 +438,15 @@ Status TransferDate64(RecordReader* reader, MemoryPool* pool,
     *out_ptr++ = static_cast<int64_t>(values[i]) * kMillisecondsPerDay;
   }
 
-  *out = std::make_shared<::arrow::Date64Array>(
-      type, length, std::move(data), reader->ReleaseIsValid(), reader->null_count());
+  if (field->nullable()) {
+    *out = std::make_shared<::arrow::Date64Array>(field->type(), length, std::move(data),
+                                                  reader->ReleaseIsValid(),
+                                                  reader->null_count());
+  } else {
+    *out =
+        std::make_shared<::arrow::Date64Array>(field->type(), length, std::move(data),
+                                               /*null_bitmap=*/nullptr, /*null_count=*/0);
+  }
   return Status::OK();
 }
 
@@ -409,22 +455,28 @@ Status TransferDate64(RecordReader* reader, MemoryPool* pool,
 
 Status TransferDictionary(RecordReader* reader,
                           const std::shared_ptr<DataType>& logical_value_type,
-                          std::shared_ptr<ChunkedArray>* out) {
+                          bool nullable, std::shared_ptr<ChunkedArray>* out) {
   auto dict_reader = dynamic_cast<DictionaryRecordReader*>(reader);
   DCHECK(dict_reader);
   *out = dict_reader->GetResult();
   if (!logical_value_type->Equals(*(*out)->type())) {
     ARROW_ASSIGN_OR_RAISE(*out, (*out)->View(logical_value_type));
   }
+  if (!nullable) {
+    ::arrow::ArrayVector chunks = (*out)->chunks();
+    ReconstructChunksWithoutNulls(&chunks);
+    *out = std::make_shared<ChunkedArray>(std::move(chunks), logical_value_type);
+  }
   return Status::OK();
 }
 
 Status TransferBinary(RecordReader* reader, MemoryPool* pool,
-                      const std::shared_ptr<DataType>& logical_value_type,
+                      const std::shared_ptr<Field>& logical_type_field,
                       std::shared_ptr<ChunkedArray>* out) {
   if (reader->read_dictionary()) {
     return TransferDictionary(
-        reader, ::arrow::dictionary(::arrow::int32(), logical_value_type), out);
+        reader, ::arrow::dictionary(::arrow::int32(), logical_type_field->type()),
+        logical_type_field->nullable(), out);
   }
   ::arrow::compute::ExecContext ctx(pool);
   ::arrow::compute::CastOptions cast_options;
@@ -434,14 +486,18 @@ Status TransferBinary(RecordReader* reader, MemoryPool* pool,
   DCHECK(binary_reader);
   auto chunks = binary_reader->GetBuilderChunks();
   for (auto& chunk : chunks) {
-    if (!chunk->type()->Equals(*logical_value_type)) {
+    if (!chunk->type()->Equals(*logical_type_field->type())) {
       // XXX: if a LargeBinary chunk is larger than 2GB, the MSBs of offsets
       // will be lost because they are first created as int32 and then cast to int64.
       ARROW_ASSIGN_OR_RAISE(
-          chunk, ::arrow::compute::Cast(*chunk, logical_value_type, cast_options, &ctx));
+          chunk,
+          ::arrow::compute::Cast(*chunk, logical_type_field->type(), cast_options, &ctx));
     }
   }
-  *out = std::make_shared<ChunkedArray>(chunks, logical_value_type);
+  if (!logical_type_field->nullable()) {
+    ReconstructChunksWithoutNulls(&chunks);
+  }
+  *out = std::make_shared<ChunkedArray>(std::move(chunks), logical_type_field->type());
   return Status::OK();
 }
 
@@ -586,7 +642,7 @@ template <
     typename = ::arrow::enable_if_t<std::is_same<ParquetIntegerType, Int32Type>::value ||
                                     std::is_same<ParquetIntegerType, Int64Type>::value>>
 static Status DecimalIntegerTransfer(RecordReader* reader, MemoryPool* pool,
-                                     const std::shared_ptr<DataType>& type, Datum* out) {
+                                     const std::shared_ptr<Field>& field, Datum* out) {
   // Decimal128 and Decimal256 are only Arrow constructs.  Parquet does not
   // specifically distinguish between decimal byte widths.
   // Decimal256 isn't relevant here because the Arrow-Parquet C++ bindings never
@@ -595,7 +651,7 @@ static Status DecimalIntegerTransfer(RecordReader* reader, MemoryPool* pool,
   // way an integer column could be construed as Decimal256 is if an arrow
   // schema was stored as metadata in the file indicating the column was
   // Decimal256. The current Arrow-Parquet C++ bindings will never do this.
-  DCHECK(type->id() == ::arrow::Type::DECIMAL128);
+  DCHECK(field->type()->id() == ::arrow::Type::DECIMAL128);
 
   const int64_t length = reader->values_written();
 
@@ -606,7 +662,7 @@ static Status DecimalIntegerTransfer(RecordReader* reader, MemoryPool* pool,
 
   const auto values = reinterpret_cast<const ElementType*>(reader->values());
 
-  const auto& decimal_type = checked_cast<const ::arrow::DecimalType&>(*type);
+  const auto& decimal_type = checked_cast<const ::arrow::DecimalType&>(*field->type());
   const int64_t type_length = decimal_type.byte_width();
 
   ARROW_ASSIGN_OR_RAISE(auto data, ::arrow::AllocateBuffer(length * type_length, pool));
@@ -622,12 +678,12 @@ static Status DecimalIntegerTransfer(RecordReader* reader, MemoryPool* pool,
     decimal.ToBytes(out_ptr);
   }
 
-  if (reader->nullable_values()) {
+  if (reader->nullable_values() && field->nullable()) {
     std::shared_ptr<ResizableBuffer> is_valid = reader->ReleaseIsValid();
-    *out = std::make_shared<Decimal128Array>(type, length, std::move(data), is_valid,
-                                             reader->null_count());
+    *out = std::make_shared<Decimal128Array>(field->type(), length, std::move(data),
+                                             is_valid, reader->null_count());
   } else {
-    *out = std::make_shared<Decimal128Array>(type, length, std::move(data));
+    *out = std::make_shared<Decimal128Array>(field->type(), length, std::move(data));
   }
   return Status::OK();
 }
@@ -640,43 +696,47 @@ static Status DecimalIntegerTransfer(RecordReader* reader, MemoryPool* pool,
 ///    representing the high and low bits of each decimal value.
 template <typename DecimalArrayType, typename ParquetType>
 Status TransferDecimal(RecordReader* reader, MemoryPool* pool,
-                       const std::shared_ptr<DataType>& type, Datum* out) {
+                       const std::shared_ptr<Field>& field, Datum* out) {
   auto binary_reader = dynamic_cast<BinaryRecordReader*>(reader);
   DCHECK(binary_reader);
   ::arrow::ArrayVector chunks = binary_reader->GetBuilderChunks();
   for (size_t i = 0; i < chunks.size(); ++i) {
     std::shared_ptr<Array> chunk_as_decimal;
     auto fn = &DecimalConverter<DecimalArrayType, ParquetType>::ConvertToDecimal;
-    RETURN_NOT_OK(fn(*chunks[i], type, pool, &chunk_as_decimal));
+    RETURN_NOT_OK(fn(*chunks[i], field->type(), pool, &chunk_as_decimal));
     // Replace the chunk, which will hopefully also free memory as we go
     chunks[i] = chunk_as_decimal;
   }
-  *out = std::make_shared<ChunkedArray>(chunks, type);
+  if (!field->nullable()) {
+    ReconstructChunksWithoutNulls(&chunks);
+  }
+  *out = std::make_shared<ChunkedArray>(chunks, field->type());
   return Status::OK();
 }
 
 }  // namespace
 
-#define TRANSFER_INT32(ENUM, ArrowType)                                              \
-  case ::arrow::Type::ENUM: {                                                        \
-    Status s = TransferInt<ArrowType, Int32Type>(reader, pool, value_type, &result); \
-    RETURN_NOT_OK(s);                                                                \
+#define TRANSFER_INT32(ENUM, ArrowType)                                               \
+  case ::arrow::Type::ENUM: {                                                         \
+    Status s = TransferInt<ArrowType, Int32Type>(reader, pool, value_field, &result); \
+    RETURN_NOT_OK(s);                                                                 \
   } break;
 
-#define TRANSFER_INT64(ENUM, ArrowType)                                              \
-  case ::arrow::Type::ENUM: {                                                        \
-    Status s = TransferInt<ArrowType, Int64Type>(reader, pool, value_type, &result); \
-    RETURN_NOT_OK(s);                                                                \
+#define TRANSFER_INT64(ENUM, ArrowType)                                               \
+  case ::arrow::Type::ENUM: {                                                         \
+    Status s = TransferInt<ArrowType, Int64Type>(reader, pool, value_field, &result); \
+    RETURN_NOT_OK(s);                                                                 \
   } break;
 
-Status TransferColumnData(RecordReader* reader, std::shared_ptr<DataType> value_type,
+Status TransferColumnData(RecordReader* reader, const std::shared_ptr<Field>& value_field,
                           const ColumnDescriptor* descr, MemoryPool* pool,
                           std::shared_ptr<ChunkedArray>* out) {
   Datum result;
   std::shared_ptr<ChunkedArray> chunked_result;
-  switch (value_type->id()) {
+  switch (value_field->type()->id()) {
     case ::arrow::Type::DICTIONARY: {
-      RETURN_NOT_OK(TransferDictionary(reader, value_type, &chunked_result));
+      RETURN_NOT_OK(TransferDictionary(reader, value_field->type(),
+                                       value_field->nullable(), &chunked_result));
       result = chunked_result;
     } break;
     case ::arrow::Type::NA: {
@@ -687,10 +747,10 @@ Status TransferColumnData(RecordReader* reader, std::shared_ptr<DataType> value_
     case ::arrow::Type::INT64:
     case ::arrow::Type::FLOAT:
     case ::arrow::Type::DOUBLE:
-      result = TransferZeroCopy(reader, value_type);
+      result = TransferZeroCopy(reader, value_field);
       break;
     case ::arrow::Type::BOOL:
-      RETURN_NOT_OK(TransferBool(reader, pool, &result));
+      RETURN_NOT_OK(TransferBool(reader, value_field->nullable(), pool, &result));
       break;
       TRANSFER_INT32(UINT8, ::arrow::UInt8Type);
       TRANSFER_INT32(INT8, ::arrow::Int8Type);
@@ -703,33 +763,33 @@ Status TransferColumnData(RecordReader* reader, std::shared_ptr<DataType> value_
       TRANSFER_INT64(TIME64, ::arrow::Time64Type);
       TRANSFER_INT64(DURATION, ::arrow::DurationType);
     case ::arrow::Type::DATE64:
-      RETURN_NOT_OK(TransferDate64(reader, pool, value_type, &result));
+      RETURN_NOT_OK(TransferDate64(reader, pool, value_field, &result));
       break;
     case ::arrow::Type::FIXED_SIZE_BINARY:
     case ::arrow::Type::BINARY:
     case ::arrow::Type::STRING:
     case ::arrow::Type::LARGE_BINARY:
     case ::arrow::Type::LARGE_STRING: {
-      RETURN_NOT_OK(TransferBinary(reader, pool, value_type, &chunked_result));
+      RETURN_NOT_OK(TransferBinary(reader, pool, value_field, &chunked_result));
       result = chunked_result;
     } break;
     case ::arrow::Type::DECIMAL128: {
       switch (descr->physical_type()) {
         case ::parquet::Type::INT32: {
           auto fn = DecimalIntegerTransfer<Int32Type>;
-          RETURN_NOT_OK(fn(reader, pool, value_type, &result));
+          RETURN_NOT_OK(fn(reader, pool, value_field, &result));
         } break;
         case ::parquet::Type::INT64: {
           auto fn = &DecimalIntegerTransfer<Int64Type>;
-          RETURN_NOT_OK(fn(reader, pool, value_type, &result));
+          RETURN_NOT_OK(fn(reader, pool, value_field, &result));
         } break;
         case ::parquet::Type::BYTE_ARRAY: {
           auto fn = &TransferDecimal<Decimal128Array, ByteArrayType>;
-          RETURN_NOT_OK(fn(reader, pool, value_type, &result));
+          RETURN_NOT_OK(fn(reader, pool, value_field, &result));
         } break;
         case ::parquet::Type::FIXED_LEN_BYTE_ARRAY: {
           auto fn = &TransferDecimal<Decimal128Array, FLBAType>;
-          RETURN_NOT_OK(fn(reader, pool, value_type, &result));
+          RETURN_NOT_OK(fn(reader, pool, value_field, &result));
         } break;
         default:
           return Status::Invalid(
@@ -741,11 +801,11 @@ Status TransferColumnData(RecordReader* reader, std::shared_ptr<DataType> value_
       switch (descr->physical_type()) {
         case ::parquet::Type::BYTE_ARRAY: {
           auto fn = &TransferDecimal<Decimal256Array, ByteArrayType>;
-          RETURN_NOT_OK(fn(reader, pool, value_type, &result));
+          RETURN_NOT_OK(fn(reader, pool, value_field, &result));
         } break;
         case ::parquet::Type::FIXED_LEN_BYTE_ARRAY: {
           auto fn = &TransferDecimal<Decimal256Array, FLBAType>;
-          RETURN_NOT_OK(fn(reader, pool, value_type, &result));
+          RETURN_NOT_OK(fn(reader, pool, value_field, &result));
         } break;
         default:
           return Status::Invalid(
@@ -755,16 +815,16 @@ Status TransferColumnData(RecordReader* reader, std::shared_ptr<DataType> value_
 
     case ::arrow::Type::TIMESTAMP: {
       const ::arrow::TimestampType& timestamp_type =
-          checked_cast<::arrow::TimestampType&>(*value_type);
+          checked_cast<::arrow::TimestampType&>(*value_field->type());
       if (descr->physical_type() == ::parquet::Type::INT96) {
         RETURN_NOT_OK(
-            TransferInt96(reader, pool, value_type, &result, timestamp_type.unit()));
+            TransferInt96(reader, pool, value_field, &result, timestamp_type.unit()));
       } else {
         switch (timestamp_type.unit()) {
           case ::arrow::TimeUnit::MILLI:
           case ::arrow::TimeUnit::MICRO:
           case ::arrow::TimeUnit::NANO:
-            result = TransferZeroCopy(reader, value_type);
+            result = TransferZeroCopy(reader, value_field);
             break;
           default:
             return Status::NotImplemented("TimeUnit not supported");
@@ -773,7 +833,7 @@ Status TransferColumnData(RecordReader* reader, std::shared_ptr<DataType> value_
     } break;
     default:
       return Status::NotImplemented("No support for reading columns of type ",
-                                    value_type->ToString());
+                                    value_field->type()->ToString());
   }
 
   if (result.kind() == Datum::ARRAY) {
